#include "mainwindow.h"

vector<double> MainWindow::myVector = vector<double>();
vector<QString> MainWindow::myLine = vector<QString>();

MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::MainWindow)
{
    ui->setupUi(this);

    myOutputs = vector< vector<double> >();

    myRenderArea = new RenderArea(400, 510);
    myLogWindow = new LogWindow();

    myRenderArea->setSceneRect(QRectF(0, 0, 399, 509));
    ui->graphicsView->setScene(myRenderArea);

    QStringList options;
    options << tr("Nmero #0") << tr("Nmero #1") << tr("Nmero #2") <<
               tr("Nmero #3") << tr("Nmero #4") << tr("Nmero #5") <<
               tr("Nmero #6") << tr("Nmero #7") << tr("Nmero #8") <<
               tr("Nmero #9");

    ui->comboBox->addItems(options);

    connect(ui->buttonDraw, SIGNAL(clicked()), this, SLOT(drawMatrix()));
    connect(ui->buttonClear, SIGNAL(clicked()), this, SLOT(clearMatrix()));
    connect(ui->buttonLearn, SIGNAL(clicked()), this, SLOT(learnPattern()));
    connect(ui->buttonRecogPattern, SIGNAL(clicked()), this, SLOT(recognizePattern()));

    setWindowTitle(tr("Reconhecimento ptico com Redes Neurais"));
}

MainWindow::~MainWindow()
{
    delete ui;
}

void MainWindow::clearMatrix()
{
    myRenderArea->clear();
    this->drawMatrix();
    ui->buttonLearn->setEnabled(true);
}

void MainWindow::learnPattern() {
    vector<double> output(10, -1);
    int sizeLayer = ui->lineEditX->text().toInt() * ui->lineEditY->text().toInt();

    myNetwork.makeNetwork( myVector, sizeLayer , 1, 10 );

    switch(ui->comboBox->currentIndex()) {
        case 0: output[0] = 1;
                myNetwork.setExpectedOutputs(0, output); break;
        case 1: output[1] = 1;
                myNetwork.setExpectedOutputs(1, output); break;
        case 2: output[2] = 1;
                myNetwork.setExpectedOutputs(2, output); break;
        case 3: output[3] = 1;
                myNetwork.setExpectedOutputs(3, output); break;
        case 4: output[4] = 1;
                myNetwork.setExpectedOutputs(4, output); break;
        case 5: output[5] = 1;
                myNetwork.setExpectedOutputs(5, output); break;
        case 6: output[6] = 1;
                myNetwork.setExpectedOutputs(6, output); break;
        case 7: output[7] = 1;
                myNetwork.setExpectedOutputs(7, output); break;
        case 8: output[8] = 1;
                myNetwork.setExpectedOutputs(8, output); break;
        case 9: output[9] = 1;
                myNetwork.setExpectedOutputs(9, output); break;
    }

    vector<double> auxoutput;
    for( int i = 0;  i < 10; i++ )
        auxoutput = myNetwork.startProcess(myVector);

    for( size_t i = 0; i < auxoutput.size(); i++ )
        myLine.push_back(QString::number(auxoutput[i]));

    myOutputs.push_back(auxoutput);

    myVector.clear();
    ui->buttonLearn->setEnabled(false);

    QMessageBox::about(this, "Alerta!", tr("Padro salvo"));
}

void MainWindow::recognizePattern()
{
    for(size_t i = 0 ; i < myNetwork.myLayers[0].size() ; i++)
    {
        myNetwork.myLayers[0][i].changeInput(myVector);
    }
    vector<Neuron> myNeuronOutput = myNetwork.propagation(myVector);

    for( size_t i = 0; i < myNeuronOutput.size(); i++ ) {
        cout << myNeuronOutput[i].getOutput() << endl;
    }
    myVector.clear();
}

void MainWindow::drawMatrix()
{
    int x = ui->lineEditX->text().toInt();
    int y = ui->lineEditY->text().toInt();

    myVector.resize(x*y, -1);
    myRenderArea->clear();
    myRenderArea->drawRects(x, y);
}

void MainWindow::on_actionAbout_triggered()
{
    DialogAbout myDialogAbout;
    myDialogAbout.setModal(true);
    myDialogAbout.exec();
}

void MainWindow::on_actionLogs_triggered()
{
    myLogWindow->addTabs(LogWindow::index++);
    myLogWindow->show();
    myLine.clear();
}

