#include "network.h"

#define TAXA 0.5

Network::Network()
{
    cout << "CREATE: Network" << endl;
    myLayers = vector < vector <Neuron> > ();
    //myDefaultInputs = multimap<int, Matrix>  ();
    ExpectedOutputs = vector < vector <double> > ();
}

Network::~Network()
{
    myLayers.clear();
    //myDefaultInputs.clear();
    ExpectedOutputs.clear();
}

void Network::makeDefaultInputs()
{
    /*cout << "START : makeDefaultInputs" << endl;

	/// Ponteiro de funes que retornam matrizes com os respectivos padres de entrada
	Matrix (*MY_DEFAULTS[10]) ( ) = {DEFAULT_0, DEFAULT_1, DEFAULT_2, DEFAULT_3, DEFAULT_4, DEFAULT_5, DEFAULT_6, DEFAULT_7, DEFAULT_8, DEFAULT_9};
    multimap<int, Matrix>::iterator it;

    /// Inserindo no multimap os 10 padres de entrada
	for(register int i = 0 ; i < SIZE_DEFAULTS ; i++)
		Network::myDefaultInputs.insert( pair<int, Matrix > (i, (*MY_DEFAULTS[i]) () ) );

    for(it = myDefaultInputs.begin() ; it != myDefaultInputs.end() ; it++ )
    {
        for(register int j = 0 ; j < 7 ; j++)
            for(register int k = 0 ; k < 5 ; k++)
                (*it).second.setNumber(j,k,0);
        //(*it).second.printMatrix();
    }
    */
}

void Network::makeNetwork(vector<double> defaults, int sizeFirstLayer,int sizeHiddenLayer, int sizeOutputLayer)
{
    if(ExpectedOutputs.size() == 0)
    {
        cout << "CRIANDO ESTRUTURA DA REDE NEURAL E PESOS SINPTICOS ALEATRIOS" << endl;
        cout << "ATRIBUIINDO A PRIMEIRA ENTRADA DA REDE NEURAL" << endl;
        vector <Neuron> firstLayer(sizeFirstLayer);
        vector <Neuron> hiddenLayer(20);
        vector <Neuron> outputLayer(sizeOutputLayer);

        ///inicializando layer da camada de entrada
        myLayers.push_back( firstLayer );
        ///inicializando layers da camada oculta
        for(register int i = 0 ; i < sizeHiddenLayer ; i++)
            myLayers.push_back( hiddenLayer );
        ///inicializando layer da camada de saida
        myLayers.push_back( outputLayer );

        ///atribuindo vector(defaults) de entradas para cada neurnio da PRIMEIRA CAMADA
        for(size_t i = 0 ; i < myLayers[0].size() ; i++)
        {
            myLayers[0][i].setInputs(defaults);
            myLayers[0][i].setLoadBias();
        }

        ///atribuindo vector(vazio) de camadas para cada neurnio das demais camadas
        for(size_t i = 1 ; i < myLayers.size() ; i++)
            for(size_t j = 0 ; j < myLayers[i].size() ; j++)
            {
                myLayers[i][j].setInputs( vector<double> ( myLayers[i-1].size() ,0) );
                myLayers[i][j].setLoadBias();
            }

        ///inicializando sada esperada para os neurnios da ultima camada
        //for(size_t i = 0 ; i < myLayers.size() ; i++)
        for(size_t i = 0 ; i < 10 ; i++)
            ExpectedOutputs.push_back( vector<double>(10) );

        /// imprimindo rede neural, este proceso  necessrio para fins de teste:
        /// OBSERVA-SE A ORGANIZAO DA CAMADA DE ENTRADA E PESOS SINPTICOS NO INICIO DO PROCESSO DE APRENDIZADO
        //Network::printNeuralNetwork();
    }
    else
    {
        cout << "PROXIMA ENTRADA DA REDE" << endl;
        ///atribuindo vector(defaults) de entradas para cada neurnio da PRIMEIRA CAMADA
        for(size_t i = 0 ; i < myLayers[0].size() ; i++)
        {
            myLayers[0][i].changeInput(defaults);
        }
    }
}

void Network::makeManuallyNetwork(vector<double> defaults)
{
}

void Network::setExpectedOutputs(int index,vector<double> Expected)
{
    for(size_t i = 0 ; i < ExpectedOutputs[index].size() ; i++)
        ExpectedOutputs[index][i] = Expected[i];
}

int Network::getSizeExpectedOutputs()
{
    return ExpectedOutputs.size();
}

void Network::printNeuralNetwork()
{
    cout << endl << endl << "REDE NEURAL" << endl << endl;
    ///imprimindo toda a rede neural
    for(size_t i = 0 ; i < myLayers.size() ; i++)
    {
        cout << "LAYER " << i << endl << endl;
        for(size_t j = 0 ; j < myLayers[i].size() ; j++)
        {
            cout << "NEURON " << j << " = " << myLayers[i][j].sigmoidFunction() << endl;
            myLayers[i][j].print();
        }
        cout << endl << endl;
    }
}

void Network::printVector(vector<double> myVector)
{
    for( size_t i = 0; i < myVector.size(); i++)
        cout << myVector[i] << " ";
    cout << endl;
}

vector<double> Network::startProcess(vector<double> inputs)
{
    vector<double> output;
    vector<Neuron> neurons;

    ///atribuindo vector(inputs) de entradas para cada neurnio da PRIMEIRA CAMADA
    for(size_t i = 0 ; i < myLayers[0].size() ; i++)
        myLayers[0][i].changeInput(inputs);

    this->propagation(inputs);
    this->backPropagation();
    this->readjustsWeights();

    /// vetor de neurnios da ULTIMA CAMADA
    neurons = myLayers[ myLayers.size()-1 ];
    /// obtendo as respectivas sadas(double [-1,1] ) de cada neurnio da ultima camada
    for(size_t i = 0 ; i < neurons.size() ; i++)
        output.push_back( neurons[i].sigmoidFunction() );

    return output;
}

vector<Neuron> Network::propagation(vector<double> inputs)
{
    for(size_t i = 0 ; i < myLayers.size() ; i++)
    {
        for(size_t j = 0 ; j < myLayers[i].size() ; j++)
        {
            ///calculando sadas da camada i
            myLayers[i][j].sumFunction();
            //cout << myLayers[i][j].sigmoidFunction() << endl;
        }
        ///repassando sadas da camada i para as entradas da camada i+1
        //     bias   input
        //pair<double, double>
        for(size_t j = 0 ; j < myLayers[i].size() ; j++)
        {
            if( i < myLayers.size()-1 )
            {
                for(size_t k = 0; k < myLayers[i+1].size() ; k++)
                    myLayers[i+1][k].addBuffer( myLayers[i][j].sigmoidFunction() );
            }
        }
        if( i < myLayers.size()-1 )
        {
            for(size_t k = 0; k < myLayers[i+1].size() ; k++)
                myLayers[i+1][k].startAssociation();
        }
        cout << endl;
    }
    Network::printNeuralNetwork();
    return myLayers[ myLayers.size() - 1 ];
}

void Network::backPropagation()
{
    double error;
    double y;
    size_t size = myLayers[ static_cast<int> (myLayers.size()-1) ].size();
    int index   = static_cast<int> (myLayers.size()-1);

    //cout << "TAM" << endl;
    //cout << size << "  " << index << endl;
    //cout << ExpectedOutputs[ size ].size() << endl;

    cout << "EXPECTED OUTPUT" << endl;
    for(size_t i = 0 ; i  < ExpectedOutputs.size() ; i++)
    {
        for(size_t j = 0 ; j < ExpectedOutputs[i].size() ; j++)
            cout << ExpectedOutputs[ i ][j] << " " ;
            //cout << ExpectedOutputs[ i ][j] << endl ;
        //cout << endl << endl << endl;
        cout << endl;
    }
    cout << endl << endl;

    ///calculando erro e delta para a ULTIMA CAMADA
    for(size_t i = 0 ; i < size ; i++)
    {
        ///sada de um neurnio
        y = myLayers[ index ][i].sigmoidFunction();
        cout << "Y" << " [" << i << "]" << " "<< y << endl ;

        ///erro de um neurnio
        error = ExpectedOutputs[ 0 ][i] - y;
        ///setter para a sada esperada de um neurnio
        myLayers[ index ][i].setExpectedOutput( ExpectedOutputs[ 0 ][i] );
        ///setter para o erro de um neurnio
        myLayers[ index ][i].setError(error);
        ///setter para o delta de um neurnio
        myLayers[ index ][i].setDelta( (1.0-pow(y,2))*error );
    }

    cout << endl << endl;
    cout << "SAIDA ESPERADA" << setw(10);
    cout << "Y" << setw(25);
    cout << "ERRO" << setw(20);
    cout << "DELTA" << endl;

    for(size_t i = 0 ; i < size ; i++)
    {
        cout << myLayers[ index ][i].getExpectedOutput() << setw(30);
        cout << myLayers[ index ][i].sigmoidFunction()   << setw(20);
        cout << myLayers[ index ][i].getError()          << setw(20);
        cout << myLayers[ index ][i].getDelta()          << endl;
    }

    ///calculando erro e delta da CAMADA OCULTA
    cout << endl << endl;
    for( size_t i = myLayers.size()-2 ; i != -1  ; i-- )
    {
        //cout << "LAYER " << i << endl << endl;
        for(size_t j = 0 ; j < myLayers[i].size() ; j++)
        {
            double value = 0;
            //cout << "NEURON- " << i << " = " << myLayers[i][j].sigmoidFunction() << endl << endl;
            for(size_t k = 0 ; k < myLayers[i+1].size() ; k++)
            {
                //cout << "NEURON+ " << k << " = " << myLayers[i+1][k].sigmoidFunction() << endl;
                //cout << myLayers[i+1][k].deltaWeight(j) << endl;
                //double var = myLayers[i+1][k].deltaWeight(k);
                value += myLayers[i+1][k].deltaWeight(j);
                //cout << endl;
            }
            //cout << "#" << value << endl;
            myLayers[i][j].setError( value );
            double y = myLayers[i][j].sigmoidFunction();
            myLayers[i][j].setDelta( (1.0-pow(y,2))*value );
            //cout << endl << endl;
            //cout << myLayers[i][j].deltaWeight(j) << endl;
        }
        //myLayers[i][j].print();
    }

    for(size_t i = 0 ; i < myLayers.size() ; i++)
    {
        cout << "LAYER " << i << endl << endl;
        for(size_t j = 0 ; j < myLayers[i].size() ; j++)
            cout << "ERROR NEURON " << j << " = " << myLayers[i][j].getError() << setw(30) << "DELTA NEURON " << j << " = " << myLayers[i][j].getDelta() << endl;
        cout << endl << endl;
    }
}
void Network::readjustsWeights()
{
    ///reajustando pesos sinpticos
    for(size_t i = 0 ; i < myLayers.size() ; i++)
    {
        for(size_t j = 0 ; j < myLayers[i].size() ; j++)
        {
            ///reajustando pesos sinpticos dos baias
            myLayers[i][j].setBias( myLayers[i][j].getMyLoadBias() + ( TAXA*1.0*myLayers[i][j].getDelta() ) );
            for(size_t k = 0 ; k < myLayers[i].size() ; k++)
            {
                //cout << myLayers[i][j].getXi(k) << " " << myLayers[i][j].getWi(k) << endl;
                ///reajustando pesos sinpticos para CADA NEURONIO
                myLayers[i][j].changeWeight(k, myLayers[i][j].getWi(k) + ( TAXA*myLayers[i][j].getXi(k)*myLayers[i][j].getDelta() ) );
            }
            //cout << endl;
            //" " << TAXA << " " << myLayers[i][j].getXi(i) << " " << myLayers[i][j].getDelta() << endl ;
            //myLayers[i][j].changeWeight(k, myLayers[i][j].getWi(j) + ( TAXA*myLayers[i][j].getXi(j)*myLayers[i][j].getDelta() ) );
            //myLayers[i][j].print();
        }
    }
    Network::printNeuralNetwork();
}
