#ifndef NETWORK_H
#define NETWORK_H

#define LENGTH_X 7
#define LENGTH_Y 5
#include <map>
#include <vector>
#include "neuron.h"
#include "logwindow.h"

#include <stdlib.h>
#include <stdio.h>

using namespace std;

class Network
{
    public:
            vector < vector <Neuron> > myLayers;

            /**
            * Construtor
            * Inicializa layers e defaultInputs
            * @fn NeuralNetwork()
            * @param void Nenhum
            */
            Network();

            /**
            * Destrutor
            * Despacho de memria
            * @fn ~NeuralNetwork()
            */
            ~Network();

            /**
            * Inicializa o multimap myDefaultInputs
            * O multimap myDefaultInputs representa as entradas padres no formato Matrix
            * @fn void makeDefaultInputs()
            * @param void Nenhum
            */
            void makeDefaultInputs();

            /**
            * Inicializa a rede neural myLayers
            * As entradas so repassadas para a primeira camada, seguidamente atribui-se a todos os pesos sinpticos valores aleatrios.
            * Por fim atribui-se ao vetor de entrada de cada neurnio(no inclui a camada de entrada) o valor "zero" seguido de um setter aleatrio para o bias.
            * @fn void makeNetwork(vector<double>, int, int, int )
            * @param void Nenhum
            */
            void makeNetwork(vector<double>, int, int, int );

            /**
            * Inicializa a rede neural myLayers
            * As entradas so repassadas para a primeira camada, seguidamente atribui-se manualmente pesos sinpticos para cada neurnio de cada camada(incluindo camada de entrada) na forma de um vetor
            * Por fim atribui-se manualmente ao vetor de entrada de cada neurnio o valor "zero" seguido de um setter manual para o bias.
            * @fn void makeManuallyNetwork(vector<double>);
            * @param void Nenhum
            */
            void makeManuallyNetwork(vector<double>);

            /**
            * Inicializa o processo de treinamento para um padro de entrada
            * Recebe um vetor de entrada e retorna um vetor de sada
            * @fn vector<double> startProcess(vector<double>);
            * @param vector<double> inputs contendo um vetor de entrada
            * @return vector<double> outputs contendo um vetor de sada
            */
            vector<double> startProcess(vector<double>);
            vector<Neuron> propagation(vector<double>);
            void backPropagation();
            void readjustsWeights();
            void printNeuralNetwork();
            void printVector(vector<double>);
            void setExpectedOutputs(int,vector<double>);
            int getSizeExpectedOutputs();
    private:
            vector < vector <double> > ExpectedOutputs;
            //multimap<int, vector<Neuron> > myLayers;
            //multimap<int, Matrix> myDefaultInputs;
            //Matrix (*MY_DEFAULTS[10]) ( );
};

#endif
