#include "neuron.h"

Neuron::Neuron()
{
    this->setLoadBias();
    this->setInputs(vector<double>());
    myActFunction = LINEAR;
}

Neuron::Neuron(Neuron::ActivationFunction func)
{
    this->setActivationFunction(func);
    myValueSumFunction = 0;
    this->setLoadBias();
}

Neuron::Neuron(vector<double> inputs, ActivationFunction func)
{
    this->setActivationFunction(func);
    this->setLoadBias();

    vector<double>::iterator it;
    double sum;

    for(it = inputs.begin(); it != inputs.end(); it++) {
        double num = this->randDouble(-1.0, 1.0);
        sum += ( (*it) * num );
    }
    // Soma com o bias
    sum = sum + myLoadBias;

    // Setter para as entradas e valor da fun&ccedil;&atilde;o soma
    myValueSumFunction = sum;
    this->setInputs(inputs);
}

Neuron::~Neuron()
{
    myInputs.clear();
    MYINPUTS.clear();
}

void Neuron::sumFunction(multimap<double, double> inputs)
{
    multimap<double, double>::iterator it;
    double sum = 0;

    for(it = inputs.begin(); it != inputs.end(); it++) {
        sum += (it->first * it->second);
    }
    // Soma com o bias
    sum = sum + myLoadBias;

    // Setter para as entradas e valor da fun&ccedil;&atilde;o soma
    myValueSumFunction = sum;
}

void Neuron::sumFunction()
{
    vector < pair <double,double> >::iterator it2;
    multimap<double, double>::iterator it;
    double sum = 0;

    for(it = myInputs.begin(), it2 = MYINPUTS.begin() ; it != myInputs.end(); it++ , it2++)
        //sum += ( (it2->first * it2->second) + myLoadBias);
        //sum += ( (it->first * it->second) + myLoadBias);
        sum += (it2->first * it2->second);

    // Setter para as entradas e valor da fun&ccedil;&atilde;o soma
    //myValueSumFunction = sum;
    myValueSumFunction = sum + myLoadBias;
}

double Neuron::stepFunction()
{
    if(myValueSumFunction >= 0.0)
        return 1;
    return 0;
}

double Neuron::linearFunction()
{
    return myValueSumFunction;
}

double Neuron::sigmoidFunction()
{
    return tanh(myValueSumFunction);
}

void Neuron::setActivationFunction(Neuron::ActivationFunction func)
{
    myActFunction = func;
}

double Neuron::randDouble(double low, double upp)
{
    double num = ( static_cast <double> (rand() % 100) / 100.0 );
    return low + num * (upp - low);

}

double Neuron::radianToDegree(double radian)
{
    return (radian * 180) / M_PI;
}

void Neuron::setInputs(vector<double> inputs)
{
    vector<double>::iterator it;

    for(it = inputs.begin(); it != inputs.end(); it++)
    {
        double num = this->randDouble(-1.0, 1.0);
        myInputs.insert( pair<double, double>(num, *it) );
        MYINPUTS.push_back( pair<double, double>(num, *it) );
    }
}

void Neuron::changeInput(vector<double> inputs)
{
    multimap<double, double>::iterator it;
    int i;

    for(it = myInputs.begin() , i = 0 ; it != myInputs.end(); it++ , i++)
    {
        (*it).second = inputs[i];
        MYINPUTS[i].second = inputs[i];
    }
}

void Neuron::changeWeight(int index, double value)
{
    MYINPUTS[index].first = value;
}

void Neuron::setInputs(vector<double> inputs, vector<double> weights, double bias)
{
    vector<double>::iterator it;
    vector<double>::iterator it2;

    for(it = inputs.begin(), it2 = weights.begin(); it != inputs.end(); it++, it2++)
    {
        myInputs.insert( pair<double, double>(*it2, *it) );
        MYINPUTS.push_back( pair<double, double>(*it2, *it) );
    }
    myLoadBias = bias;
}

void Neuron::setInput(double input)
{
    double num = this->randDouble(-1.0, 1.0);
    myInputs.insert( pair<double, double>(num, input) );
    MYINPUTS.push_back( pair<double, double>(num, input) );
}

void Neuron::setInput(double input, double load)
{
    myInputs.insert( pair<double, double>(load, input) );
    MYINPUTS.push_back( pair<double, double>(load, input) );
}

double Neuron::getValueSumFunction()
{
    return myValueSumFunction;
}

double Neuron::getOutput()
{
    double output;
    this->sumFunction();

    switch(myActFunction) {
        case LINEAR:
            output = this->linearFunction(); break;
        case STEP:
            output = this->stepFunction(); break;
        case SIGMOID:
            output = this->sigmoidFunction(); break;
    }

    return output;
}

Neuron::ActivationFunction Neuron::getActivationFunction()
{
    return myActFunction;
}

void Neuron::setLoadBias()
{
    myLoadBias = this->randDouble(-1.0, 1.0);
}

void Neuron::setBias(double bias)
{
    myLoadBias = bias;
}

void Neuron::setError(double error)
{
    myError = error;
}

void Neuron::setExpectedOutput(double expectedOutput)
{
    myExpectedOutput = expectedOutput;
}

void Neuron::setDelta(double delta)
{
    myDelta = delta;
}

const double Neuron::getError() const
{
    return myError;
}

double Neuron::getExpectedOutput() const
{
    return myExpectedOutput;
}

const double Neuron::getDelta() const
{
    return myDelta;
}

void Neuron::print()
{
    /*
    multimap<double, double>::iterator it;
    int i;
    cout << "1*[" << myLoadBias << "]" << endl;
    for(it = myInputs.begin() , i = 0 ; it != myInputs.end() ; it++ , i++)
        cout << (*it).second << "--->" << (*it).first << endl;
    */
    vector < pair <double,double> >::iterator it;
    int i;
    cout << "1*[" << myLoadBias << "]" << endl;
    for(it = MYINPUTS.begin() , i = 0 ; it != MYINPUTS.end() ; it++ , i++)
        cout << (*it).second << "--->" << (*it).first << endl;
}

multimap<double, double> Neuron::getMyInputs()
{
    return myInputs;
}

void Neuron::addBuffer(double weight )
{
    myBuffer.push_back( weight );
}

void Neuron::startAssociation()
{
    multimap<double, double>::iterator it;
    multimap<double, double> newInput;

    vector < pair <double,double> >::iterator it2;
    vector < pair <double,double> >NEWINPUT;
    int i;

    for(it = myInputs.begin() , it2 = MYINPUTS.begin() , i = 0 ; it != myInputs.end() ; it++ , it2++, i++)
    {
        NEWINPUT.push_back( pair<double,double> ( (*it2).first ,myBuffer[i] ) );
        newInput.insert( pair<double,double> ( (*it).first ,myBuffer[i] ) );
    }
    myInputs = newInput;
    MYINPUTS = NEWINPUT;
    myBuffer.clear();
}

double Neuron::deltaWeight(int index)
{
    return ( MYINPUTS[index].first * getDelta() );
}

double Neuron::getXi(int index)
{
    return MYINPUTS[index].second;
}

double Neuron::getWi(int index)
{
    return MYINPUTS[index].first;
}

double Neuron::getMyLoadBias()
{
    return myLoadBias;
}
